@@ -1,5 +1,5 @@
-# 2022.3.20 snt-mkf 
-import en,difflib,requests,sys,traceback
+# 2022.4.7 snt-mkf 
+import en,difflib,requests,sys,traceback,re
 trans_diff		= lambda src, trg:  [] if src == trg else [s for s in difflib.ndiff(src, trg) if not s.startswith('?')] #src:list, trg:list
 trans_diff_merge= lambda src, trg:  [] if src == trg else [s.strip() for s in "^".join([s for s in difflib.ndiff(src, trg) if not s.startswith('?')]).replace("^+","|+").split("^") if not s.startswith("+") ]
 mkf_input		= lambda i, snt, gec, toklist, gec_toklist, doc, diffmerge,pid=0: 	{"pid":pid, "sid":i, "snt":snt, "tok": toklist,  #"offset":-1,"len":-1,"re_sntbr":0,  normally, offset =0
@@ -7,7 +7,53 @@ mkf_input		= lambda i, snt, gec, toklist, gec_toklist, doc, diffmerge,pid=0: 	{"
 				"seg":[ ("NP", sp.start, sp.end) for sp in doc.noun_chunks] + [ (np.label_, np.start,np.end) for np in doc.ents] , 
 				"gec": gec, "diff": trans_diff_merge( toklist , gec_toklist) if diffmerge else trans_diff( toklist , gec_toklist)	}
 
-def sntmkf(dskhost:str, snt:str, gec:str, diffmerge:bool=False): 
+def mkf_inputs(valids, docs,  sntdic:dict={}, diffmerge:bool=False, getdoc=lambda snt: spacy.nlp(snt)):
+	''' valids = snts, 2022.4.7  '''
+	return [ mkf_input(i,valids[i],sntdic.get(valids[i],valids[i]), [t.text for t in doc], [t.text for t in (doc if valids[i] == sntdic.get(valids[i],valids[i]) else getdoc(sntdic.get(valids[i],valids[i]) )) ], doc, diffmerge )  for i, doc in enumerate(docs)]
+
+def adjust_by_wordlist(snt, mkf):
+	''' '''
+	for k,v in mkf.get('feedback',{}).items(): 
+		try:
+			if 'ibeg_byte' in v: del v['ibeg_byte']
+			if 'iend_byte' in v: del v['iend_byte']
+			if 'word_list' in v:
+				m = re.search(f"\\b{v['word_list']}\\b",snt) #offset = snt.find(v['word_list'])  
+				if m is not None: 
+					v['ibeg_byte'], v['iend_byte'] = m.span() # mark the first hitted  #v['ibeg_byte'] = offset  #v['iend_byte'] = offset + len(v['word_list'])
+		except Exception as ex:
+			print(">>adjust_by_wordlist Ex:", ex, "\t|", snt)
+			exc_type, exc_value, exc_traceback_obj = sys.exc_info()
+			traceback.print_tb(exc_traceback_obj)
+			#redis.r.hset(f"debug:{id}", "ex:_adjust_by_wordlist", json.dumps([ str(ex), traceback.format_tb(exc_traceback_obj)]) )
+
+def snt_mkf(snts, docs, sntdic:dict={}, ibeg_byte:bool=False, diffmerge:bool=False, batch:int=0, dskhost:str="172.17.0.1:7095") :  
+	''' multiple process, ie: batch sent count in one shot=16,  added 2022.4.7 '''
+	import grequests
+	inputs = mkf_inputs(snts, docs, sntdic, diffmerge)
+
+	if batch <= 0: # single process, tranditional way 
+		mkfs	= requests.post(f"http://{dskhost}/parser", data={"q":json.dumps(inputs).encode("utf-8")}).json()
+		if ibeg_byte: [adjust_by_wordlist(snt,mkf) for snt,mkf in zip(snts, mkfs)]
+		return dict(zip(snts, mkfs))
+
+	req_list = []
+	offset = 0 
+	while offset < len(inputs): 
+		req_list.append( grequests.post(f"http://{dskhost}/parser", data={"q":json.dumps(inputs[offset:offset+batch]).encode("utf-8")}) )
+		offset = offset + batch
+
+	res_list = grequests.map(req_list)
+	dic = {}
+	for res in res_list: 	
+		for ar in res.json(): #[{"feedback":{"_punct_space@w_punct.space_extra":{"cate":"w_punct.space_extra",
+			snt = ar.get('meta',{}).get('snt','')
+			if snt: 
+				dic[snt] = ar 
+				if ibeg_byte: adjust_by_wordlist(snt, ar)
+	return dic 
+
+def sntmkf(snt:str, gec:str, diffmerge:bool=False, dskhost:str="172.17.0.1:7095" ): 
 	''' snt -> mkf, for debug only '''
 	try:
 		doc		= spacy.nlp(snt) 
@@ -89,6 +135,14 @@ def uvirun(port):
 
 	uvicorn.run(app, host='0.0.0.0', port=port)
 
+def test():
+	from dic import essay 
+	snts	= spacy.snts(essay) 
+	docs	= [spacy.nlp(snt) for snt in snts]
+	#inputs	= mkf_inputs(snts, docs) 
+	mkfs	= snt_mkf(snts, docs, ibeg_byte=True, dskhost='dsk.jukuu.com')
+	print (len(mkfs), mkfs)
+
 if __name__ == '__main__':
 	import fire 
-	fire.Fire({ "uvirun":uvirun, "hello": lambda: sntsmkf(asdsk=False, ), 'feedbacks': lambda: feedbacks()}) 
\ No newline at end of file
+	fire.Fire({"test":test, "uvirun":uvirun, "hello": lambda: sntsmkf(asdsk=False, ), 'feedbacks': lambda: feedbacks()}) 
\ No newline at end of file
