@@ -1,5 +1,5 @@
 # 2022.3.20, usage:  import en |  spacy.snts("hello") 
-import json,spacy,os,builtins,sqlite3
+import json,spacy,os,builtins
 from spacy.tokens import DocBin,Doc,Token
 from spacy.tokenizer import Tokenizer
 from spacy.lang.char_classes import ALPHA, ALPHA_LOWER, ALPHA_UPPER, CONCAT_QUOTES, LIST_ELLIPSES, LIST_ICONS
@@ -38,19 +38,20 @@ def spacydoc(snt, use_cache=True):
 	spacydoc.db.conn.commit()
 	return doc 
 
-def cachedoc(snt, table): 
+def sqlitedoc(snt, table): 
 	'''multiple tables supported,  added 2022.3.27 '''
-	if not hasattr(cachedoc, 'conn'): 
-		cachedoc.conn =	sqlite3.connect("spacy311.sntbs", check_same_thread=False) 
-		cachedoc.conn.execute('PRAGMA synchronous=OFF')
+	import sqlite3
+	if not hasattr(sqlitedoc, 'conn'): 
+		sqlitedoc.conn =	sqlite3.connect("spacy311.sntbs", check_same_thread=False) 
+		sqlitedoc.conn.execute('PRAGMA synchronous=OFF')
 
-	cachedoc.conn.execute(f'CREATE TABLE IF NOT EXISTS {table} (key varchar(512) PRIMARY KEY, value blob)')
-	item = cachedoc.conn.execute(f'SELECT value FROM "{table}" WHERE key = ? limit 1', (key,)).fetchone()
+	sqlitedoc.conn.execute(f'CREATE TABLE IF NOT EXISTS {table} (key varchar(512) PRIMARY KEY, value blob)')
+	item = sqlitedoc.conn.execute(f'SELECT value FROM "{table}" WHERE key = ? limit 1', (key,)).fetchone()
 	if item	is not None: return spacy.frombs(item[0]) 
 
 	doc = spacy.nlp(snt) 
-	cachedoc.conn.execute(f'REPLACE	INTO {table} (key,	value) VALUES (?,?)',	(snt, spacy.tobs(doc) ))
-	cachedoc.conn.commit()
+	sqlitedoc.conn.execute(f'REPLACE	INTO {table} (key,	value) VALUES (?,?)',	(snt, spacy.tobs(doc) ))
+	sqlitedoc.conn.commit()
 	return doc 
 
 if not hasattr(spacy, 'nlp'):
@@ -58,13 +59,14 @@ if not hasattr(spacy, 'nlp'):
 	spacy.sntbr		= (inst := en.English(), inst.add_pipe("sentencizer"))[0]
 	spacy.snts		= lambda essay, trim=True: [ snt.text.strip() if trim else snt.text for snt in  spacy.sntbr(essay).sents]
 	spacy.sntpid	= lambda essay: (pid:=0, [ ( pid := pid + 1 if "\n" in snt.text else pid,  (snt.text, pid))[-1] for snt in  spacy.sntbr(essay).sents] )[-1]
+	spacy.sntpidoff	= lambda essay: (pid:=0, doc:=spacy.sntbr(essay), [ ( pid := pid + 1 if "\n" in snt.text else pid,  (snt.text, pid, doc[snt.start].idx))[-1] for snt in  doc.sents] )[-1]
 	spacy.nlp		= spacy.load('en_core_web_sm')
 	spacy.frombs	= lambda bs: list(spacy.tokens.DocBin().from_bytes(bs).get_docs(spacy.nlp.vocab))[0] if bs else None
 	spacy.tobs		= lambda doc: ( doc_bin:= spacy.tokens.DocBin(), doc_bin.add(doc), doc_bin.to_bytes())[-1]
 	spacy.tok		= lambda doc: ' '.join([t.text.strip() for t in doc])
 	spacy.nps		= lambda doc : {f"{doc[np.end-1].lemma_}/np:{np.text.lower()}" for np in doc.noun_chunks} #book/np:a book
 	spacy.trps		= lambda doc : {f"{t.dep_}_{t.head.pos_}_{t.pos_}:{t.head.lemma_} {t.lemma_}" for t in doc if t.pos_ not in ("PUNCT","PROPN","NUM","SPACE") and t.text.isalpha()} #'dobj_VERB_NOUN:open door';
-	spacy.getdoc	= lambda snt, use_cache=True: spacydoc(snt, use_cache) #lambda snt: ( bs := spacy.txn.get(snt.encode()), doc := spacy.frombs(bs) if bs else spacy.nlp(snt), spacy.txn.put(snt.encode(), spacy.tobs(doc)) if not bs else None )[1] 
+	spacy.getdoc	= lambda snt, getf=lambda snt: None : ( res := getf(snt), res if res else spacy.nlp(snt))[1] 
 	spacy.redisdoc	= lambda snt, r, prefix="": ( bs := r.get(prefix+snt), doc := spacy.frombs(bs) if bs else spacy.nlp(snt), r.setnx(prefix+snt, spacy.tobs(doc)) if not bs else None )[1] # added 2022.3.29
 	spacy.merge_nps	= spacy.nlp.create_pipe("merge_noun_chunks")
 	spacy.nlp.tokenizer = custom_tokenizer(spacy.nlp)	#nlp.tokenizer.infix_finditer = infix_re.finditer
@@ -111,7 +113,7 @@ def readline(infile, sepa=None):
 #setattr(builtins, "spacy", spacy)
 			
 if __name__	== '__main__': 
-	print(spacy.nlp("I love you."))
+	print(spacy.sntpidoff("   English   is a internationaly language which becomes importantly for modern world.  In China, English is took to be."))
 
 #c:\users\zhang\appdata\local\programs\python\python38\lib\site-packages  
-#/home/ubuntu/.local/lib/python3.8/site-packages/en
\ No newline at end of file
+#/home/ubuntu/.local/lib/python3.8/site-packages/en
